import react, { useMemo, useEffect } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import _ from 'lodash';
import { MapState, mapActions } from '../../store/map';
import { slippyActions } from '../../store/slippy';
import { lat2tile, lon2tile } from '../../lib/geo';
import Tile from './tile';

import '../../theme/map.css';

export const tileSize = 256;

const Map: react.FC<{}> = (props: {}) => {
  const dispatch = useDispatch();

  const mapState = useSelector((state: { map: MapState }) => state.map);

  console.log(`mapState: ${JSON.stringify(mapState)}`);

  const resizeHandler = () => {
    dispatch(mapActions.resize());
  };
  const debouncedResizeHandler = useMemo(
    () => _.debounce(resizeHandler, 500),
    []
  );

  useEffect(() => {
    window.addEventListener('resize', debouncedResizeHandler);
  }, []);

  const nbTilesY = _.ceil(mapState.viewport.height / tileSize) + 3;
  const nbTilesX = _.ceil(mapState.viewport.width / tileSize) + 3;
  const [tileCenterY, reminderY] = lat2tile(
    mapState.scope.center.lat,
    mapState.scope.zoom
  );
  const [tileCenterX, reminderX] = lon2tile(
    mapState.scope.center.lon,
    mapState.scope.zoom
  );
  const firstTileY = tileCenterY - _.round(nbTilesY / 2);
  const firstTileX = tileCenterX - _.round(nbTilesX / 2);
  const locationY = (tileCenterY + reminderY - firstTileY) * tileSize;
  const locationX = (tileCenterX + reminderX - firstTileX) * tileSize;
  const targetLocationY = mapState.viewport.height / 2;
  const targetLocationX = mapState.viewport.width / 2;
  const deltaY = targetLocationY - locationY;
  const deltaX = targetLocationX - locationX;

  dispatch(slippyActions.set({ scale: 1, translation: { x: deltaX, y: deltaY } }));

  return (
    <div className='tiles'>
      {_.range(nbTilesY).map((iy) => (
        <div key={'y' + iy} className='tilesRow'>
          {_.range(nbTilesX).map((ix) => (
            <Tile
              key={'x' + ix + 'y' + iy}
              iy={iy}
              ix={ix}
              x={firstTileX + ix}
              y={firstTileY + iy}
              zoom={mapState.scope.zoom}
            />
          ))}
        </div>
      ))}
    </div>
  );
};

export default Map;
