import { createSlice } from '@reduxjs/toolkit';
import { geoPoint } from '../lib/geo';

export interface MapState {
  viewport: {
    width: number;
    height: number;
  };
  scope: {
    center: geoPoint;
    zoom: number;
  };
}

const initialMapState: MapState = {
  viewport: {
    width: window.innerWidth,
    height: window.innerHeight,
  },
  scope: {
    center: { lat: -37.8372, lon: 77.5513 },
    zoom: 13,
  },
};

const mapSlice = createSlice({
  name: 'map',
  initialState: initialMapState,
  reducers: {
    resize: (state) => {
      state.viewport.height = window.innerHeight;
      state.viewport.width = window.innerWidth;
    },
  },
});

export const mapActions = mapSlice.actions;

export default mapSlice.reducer;
